/*
 * hex2com.c - convert intellec hex file to core-image format
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define	BUFFSIZE	16384
static	char	Buff[BUFFSIZE];
static	char	Sum;
static	FILE	*Ifp;



void	usage()
{
	fprintf(stderr, "Usage: hex2com [-b bias] [file1] >file2\n");
}



FILE	*efopen(char *file, char *mode)
{
	FILE	*fp;

	if ((fp = fopen(file, mode)) == NULL) {
		perror(file);
		exit(1);
	}
	return (fp);
}



int	getnib()
{
	int	c;

	c = getc(Ifp);
	if (isdigit(c))
		return (c - '0');
	else if ('A' <= c && c <= 'F')
		return (c - 'A' + 10);
	fprintf(stderr, "hex2com: illegal hex digit\n");
	exit(1);
}

int	getbyte()
{
	int	c;

	c = getnib() << 4;
	c |= getnib();
	Sum += c;
	return (c);
}

int	getword()
{
	int	c;

	c = getbyte() << 8;
	c |= getbyte();
	return (c);
}


main(int argc, char *argv[])
{
	int	c, len, first;
	char	*p, *e;
	unsigned bias, base, bufforg, maxbuff;

	bias = 0;
	while (++argv, --argc > 0 && argv[0][0] == '-') {
		switch (argv[0][1]) {
		case 'b':
			errno = 0;
			if (argv[0][2]) {
				bias = strtol(argv[0] + 2, &e, 16);
			    printf("bias=%u", bias);
			} else {
				if (argc < 0) {
					usage();
					exit(2);
				}
				argc--;
				bias = strtol(*++argv, &e, 16);
			}
			if (errno || *e != '\0') {
				fprintf(stderr, "hex2com: bad bias\n");
				exit(2);
			}
			break;
		default:
			usage();
			exit(2);
		}
	}
	if (argc > 1) {
		usage();
		exit(2);
	}
	if (argc == 1)
		Ifp = efopen(argv[0], "r");
	else
		Ifp = stdin;
	fsetbin(stdout);

	first = 1;
	bufforg = 0xffff;
	maxbuff = 0;
	for (;;) {
		while ((c = getc(Ifp)) != ':')
			if (c == EOF) {
				fprintf(stderr, "hex2com: missing end record\n");
				exit(1);
			}
		Sum = 0;
		len = getbyte();
		base = getword();
		if (getbyte() == 1)
			break;
		if (len == 0)
			break;
		if (base < bias) {
			fprintf(stderr, "hex2com: address too small at %04X\n",
					base);
			exit(1);
		}
		base -= bias;
		if (first || base < bufforg || base + len >= bufforg + BUFFSIZE) {
			if (maxbuff > 0) {
				fseek(stdout, bufforg, SEEK_SET);
				fwrite(Buff, 1, maxbuff, stdout);
			}
			bufforg = base;
			maxbuff = 0;
			memset(Buff, 0, BUFFSIZE);
			first = 0;
		}
		if (maxbuff < (base + len) - bufforg)
			maxbuff = (base + len) - bufforg;
		p = Buff + (base - bufforg);
		do {
			*p++ = getbyte();
		} while (--len);
		getbyte();
		if (Sum != 0)
			fprintf(stderr, "hex2com: check sum error at %04X\n",
					base);
	}
	if (maxbuff > 0) {
		fseek(stdout, bufforg, SEEK_SET);
		fwrite(Buff, 1, maxbuff, stdout);
	}
fin:
	fclose(Ifp);
	exit(0);
}
